'use strict';

function _asyncToGenerator(fn) { return function () { var gen = fn.apply(this, arguments); return new Promise(function (resolve, reject) { function step(key, arg) { try { var info = gen[key](arg); var value = info.value; } catch (error) { reject(error); return; } if (info.done) { resolve(value); } else { return Promise.resolve(value).then(function (value) { step("next", value); }, function (err) { step("throw", err); }); } } return step("next"); }); }; }

const promisify = require('./utils/promisify');
const resolve = promisify(require('browser-resolve'));
const builtins = require('./builtins');
const path = require('path');
const glob = require('glob');

class Resolver {
  constructor(options = {}) {
    this.options = options;
    this.cache = new Map();
  }

  resolve(filename, parent) {
    var _this = this;

    return _asyncToGenerator(function* () {
      let key = (parent ? path.dirname(parent) : '') + ':' + filename;
      if (_this.cache.has(key)) {
        return _this.cache.get(key);
      }

      if (glob.hasMagic(filename)) {
        return { path: path.resolve(path.dirname(parent), filename) };
      }

      var res = yield resolve(filename, {
        filename: parent,
        paths: _this.options.paths,
        modules: builtins,
        extensions: Object.keys(_this.options.extensions),
        packageFilter(pkg, pkgfile) {
          // Expose the path to the package.json file
          pkg.pkgfile = pkgfile;
          return pkg;
        }
      });

      if (Array.isArray(res)) {
        res = { path: res[0], pkg: res[1] };
      } else {
        res = { path: res, pkg: null };
      }

      _this.cache.set(key, res);
      return res;
    })();
  }
}

module.exports = Resolver;